@@ -2,7 +2,7 @@
 ## Reader
 
 getReaders <- function()
-    c("readDOC", "readGmane", "readPDF", "readReut21578XML", "readReut21578XMLasPlain", "readPlain", "readRCV1", "readRCV1asPlain", "readTabular", "readXML")
+    c("readDOC", "readPDF", "readReut21578XML", "readReut21578XMLasPlain", "readPlain", "readRCV1", "readRCV1asPlain", "readTabular", "readXML")
 
 prepareReader <- function(readerControl, defaultReader = NULL, ...) {
     if (is.null(readerControl$reader))
@@ -14,11 +14,13 @@ prepareReader <- function(readerControl, defaultReader = NULL, ...) {
     readerControl
 }
 
-readPlain <- FunctionGenerator(function(...) {
-    function(elem, language, id) PlainTextDocument(elem$content, id = id, language = language)
-})
+readPlain <-
+function(elem, language, id)
+    PlainTextDocument(elem$content, id = id, language = language)
+
+readXML <- FunctionGenerator(function(spec, doc) {
+    stopifnot(is.list(spec), inherits(doc, "TextDocument"))
 
-readXML <- FunctionGenerator(function(spec, doc, ...) {
     spec <- spec
     doc <- doc
     function(elem, language, id) {
@@ -36,37 +38,27 @@ readXML <- FunctionGenerator(function(spec, doc, ...) {
     }
 })
 
-readGmane <- readXML(spec = list(Author = list("node", "/item/creator"),
-                     Content = list("node", "/item/description"),
-                     DateTimeStamp = list("function", function(node)
-                     strptime(sapply(XML::getNodeSet(node, "/item/date"), XML::xmlValue),
-                              format = "%Y-%m-%dT%H:%M:%S",
-                              tz = "GMT")),
+readReut21578XML <-
+readXML(spec = list(Author = list("node", "/REUTERS/TEXT/AUTHOR"),
+                    DateTimeStamp = list("function", function(node)
+                      strptime(sapply(XML::getNodeSet(node, "/REUTERS/DATE"),
+                                      XML::xmlValue),
+                               format = "%d-%B-%Y %H:%M:%S",
+                               tz = "GMT")),
                      Description = list("unevaluated", ""),
-                     Heading = list("node", "/item/title"),
-                     ID = list("node", "/item/link"),
-                     Origin = list("unevaluated", "Gmane Mailing List Archive")),
-                     doc = PlainTextDocument())
-
-readReut21578XML <- readXML(spec = list(Author = list("node", "/REUTERS/TEXT/AUTHOR"),
-                            DateTimeStamp = list("function", function(node)
-                            strptime(sapply(XML::getNodeSet(node, "/REUTERS/DATE"), XML::xmlValue),
-                                     format = "%d-%B-%Y %H:%M:%S",
-                                     tz = "GMT")),
-                            Description = list("unevaluated", ""),
-                            Heading = list("node", "/REUTERS/TEXT/TITLE"),
-                            ID = list("attribute", "/REUTERS/@NEWID"),
-                            TOPICS = list("attribute", "/REUTERS/@TOPICS"),
-                            LEWISSPLIT = list("attribute", "/REUTERS/@LEWISSPLIT"),
-                            CGISPLIT = list("attribute", "/REUTERS/@CGISPLIT"),
-                            OLDID = list("attribute", "/REUTERS/@OLDID"),
-                            Origin = list("unevaluated", "Reuters-21578 XML"),
-                            Topics = list("node", "/REUTERS/TOPICS/D"),
-                            Places = list("node", "/REUTERS/PLACES/D"),
-                            People = list("node", "/REUTERS/PEOPLE/D"),
-                            Orgs = list("node", "/REUTERS/ORGS/D"),
-                            Exchanges = list("node", "/REUTERS/EXCHANGES/D")),
-                            doc = Reuters21578Document())
+                     Heading = list("node", "/REUTERS/TEXT/TITLE"),
+                     ID = list("attribute", "/REUTERS/@NEWID"),
+                     TOPICS = list("attribute", "/REUTERS/@TOPICS"),
+                     LEWISSPLIT = list("attribute", "/REUTERS/@LEWISSPLIT"),
+                     CGISPLIT = list("attribute", "/REUTERS/@CGISPLIT"),
+                     OLDID = list("attribute", "/REUTERS/@OLDID"),
+                     Origin = list("unevaluated", "Reuters-21578 XML"),
+                     Topics = list("node", "/REUTERS/TOPICS/D"),
+                     Places = list("node", "/REUTERS/PLACES/D"),
+                     People = list("node", "/REUTERS/PEOPLE/D"),
+                     Orgs = list("node", "/REUTERS/ORGS/D"),
+                     Exchanges = list("node", "/REUTERS/EXCHANGES/D")),
+                     doc = Reuters21578Document())
 
 readReut21578XMLasPlain <- readXML(spec = list(Author = list("node", "/REUTERS/TEXT/AUTHOR"),
                                    Content = list("node", "/REUTERS/TEXT/BODY"),
@@ -117,7 +109,9 @@ readRCV1asPlain <- readXML(spec = list(Author = list("unevaluated", ""),
                            doc = PlainTextDocument())
 
 # readDOC needs antiword installed to be able to extract the text
-readDOC <- FunctionGenerator(function(AntiwordOptions = "", ...) {
+readDOC <- FunctionGenerator(function(AntiwordOptions = "") {
+    stopifnot(is.character(AntiwordOptions))
+
     AntiwordOptions <- AntiwordOptions
     function(elem, language, id) {
         content <- system2("antiword",
@@ -127,20 +121,45 @@ readDOC <- FunctionGenerator(function(AntiwordOptions = "", ...) {
     }
 })
 
-# readPDF needs pdfinfo and pdftotext installed to extract meta data and text
-readPDF <- FunctionGenerator(function(PdftotextOptions = "", ...) {
-    PdftotextOptions <- PdftotextOptions
+readPDF <-
+FunctionGenerator(function(engine = c("xpdf", "Rpoppler", "ghostscript", "Rcampdf", "custom"),
+                           control = list(info = NULL, text = NULL))
+{
+    stopifnot(is.character(engine), is.list(control))
+
+    engine <- match.arg(engine)
+    control <- control
+
+    pdf_info <-
+        switch(engine,
+               xpdf = function(x) pdf_info_via_xpdf(x, control$info),
+               Rpoppler = Rpoppler::PDF_info,
+               ghostscript = pdf_info_via_gs,
+               Rcampdf = Rcampdf::pdf_info,
+               custom = control$info)
+
+    pdf_text <-
+        switch(engine,
+               xpdf = function(x) system2("pdftotext",
+                                          c(control$text, shQuote(x), "-"),
+                                          stdout = TRUE),
+               Rpoppler = Rpoppler::PDF_text,
+               ghostscript = pdf_text_via_gs,
+               Rcampdf = Rcampdf::pdf_text,
+               custom = control$text)
+
+    if (!is.function(pdf_info) || !is.function(pdf_text))
+        stop("invalid function for PDF extraction")
+
     function(elem, language, id) {
-        meta <- tm:::pdfinfo(elem$uri)
-        content <- system2("pdftotext",
-                           c(PdftotextOptions, shQuote(elem$uri), "-"),
-                           stdout = TRUE)
+        meta <- pdf_info(elem$uri)
+        content <- pdf_text(elem$uri)
         PlainTextDocument(content, meta$Author, meta$CreationDate, meta$Subject,
                           meta$Title, id, meta$Creator, language)
      }
 })
 
-readTabular <- FunctionGenerator(function(mapping, ...) {
+readTabular <- FunctionGenerator(function(mapping) {
     mapping <- mapping
     function(elem, language, id) {
         doc <- PlainTextDocument(id = id, language = language)
